@@ -17,8 +17,6 @@ from tempfile import NamedTemporaryFile
 
 from asteval import NameFinder, Interpreter, make_symbol_table
 
-
-
 HAS_NUMPY = False
 try:
     import numpy as np
@@ -28,6 +26,8 @@ except ImportError:
     HAS_NUMPY = False
 
 
+version_info = (version_info.major, version_info.minor)
+
 class TestCase(unittest.TestCase):
     """testing of asteval"""
     def setUp(self):
@@ -141,6 +141,22 @@ class TestEval(TestCase):
         self.istrue("a_dict['a'] == 1")
         self.istrue("a_dict['d'] == 4")
 
+    def test_dict_set_index(self):
+        """dictionary indexing"""
+        self.interp("a_dict = {'a': 1, 'b': 2, 'c': 3, 'd': 4}")
+        self.interp("a_dict['a'] = -4")
+        self.interp("a_dict['e'] = 73")
+
+        self.istrue("a_dict['a'] == -4")
+        self.istrue("a_dict['e'] == 73")
+
+        self.interp("b_dict = {}")
+        self.interp("keyname = 'a'")
+        self.interp("b_dict[keyname] = (1, -1, 'x')")
+        self.istrue("b_dict[keyname] ==  (1, -1, 'x')")
+
+
+
     def test_list_index(self):
         """list indexing"""
         self.interp("a_list = ['a', 'b', 'c', 'd', 'o']")
@@ -177,6 +193,7 @@ class TestEval(TestCase):
             self.interp("y = arange(20).reshape(4, 5)")
             self.istrue("y[:,3]  == array([3, 8, 13, 18])")
             self.istrue("y[...,1]  == array([1, 6, 11, 16])")
+            self.istrue("y[1,:] == array([5, 6, 7, 8, 9])")
             self.interp("y[...,1] = array([2, 2, 2, 2])")
             self.istrue("y[1,:] == array([5, 2, 7, 8, 9])")
 
@@ -265,6 +282,8 @@ class TestEval(TestCase):
         self.check_error(None)
         self.interp('assert n==7')
         self.check_error('AssertionError')
+        self.interp('assert n==7, "no match"')
+        self.check_error('AssertionError', 'no match')
 
     def test_for(self):
         """for loops"""
@@ -731,7 +750,7 @@ class TestEval(TestCase):
         """test function with kw args, no **kws"""
         self.interp(textwrap.dedent("""
             def fcn(x=0, y=0, z=0, t=0, square=False):
-                'test varargs function'
+                'test kwargs function'
                 out = 0
                 for i in (x, y, z, t):
                     if square:
@@ -882,8 +901,12 @@ class TestEval(TestCase):
         self.interp("9**9**9**9**9**9**9**9")
         self.check_error('RuntimeError')  # Safe, safe_pow() catches this
         self.interp(
-            "((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((1))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))")
-        self.check_error('MemoryError')  # Hmmm, this is caught, but its still concerning...
+            "x = ((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((1))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))")
+        if version_info == (3, 9):
+            self.isvalue('x', 1)
+            self.check_error(None)
+        else:
+            self.check_error('MemoryError')  # Hmmm, this is caught, but its still concerning...
         self.interp("compile('xxx')")
         self.check_error('NameError')  # Safe, compile() is not supported
 
@@ -1036,10 +1059,32 @@ class TestEval(TestCase):
         # uses the function's __name__ attribute. Partials don't have a
         # __name__ attribute, so we want to make sure that an AttributeError is
         # not raised.
-    
+
         result = aeval("sqrt(-1)")
         assert aeval.error.pop().exc == ValueError
 
+    def test_inner_return(self):
+        self.interp(textwrap.dedent("""
+            def func():
+                loop_cnt = 0
+                for i in range(5):
+                    for k in range(5):
+                        loop_cnt += 1
+                    return (i, k, loop_cnt)
+        """))
+        out = self.interp("func()")
+        assert out == (0, 4, 5)
+
+    def test_nested_break(self):
+        self.interp(textwrap.dedent("""
+        def func_w():
+            for k in range(5):
+                if k == 4:
+                    break
+                    k = 100
+            return k
+        """))
+        assert 4 == self.interp("func_w()")
 
 class TestCase2(unittest.TestCase):
     def test_stringio(self):
